<?php

namespace Drupal\Tests\watchdog_mailer\Functional;

/**
 * This class provides methods specifically for testing something.
 *
 * @group watchdog_mailer
 */
class WatchdogMailerSettingsTest extends WatchdogMailerTestBase {

  /**
   * Tests initial values.
   */
  public function testInitialValues() {
    $session = $this->assertSession();
    // Go to the settings page and check if the default settings are applied.
    $this->drupalGet('/admin/config/development/logging');
    $session->checkboxNotChecked('watchdog_mailer_enable');
    $session->fieldValueEquals('mail_addresses', '');
    $session->fieldValueEquals('mail_subject', '[Watchdog Mailer ([watchdog_mailer:channel])] "[watchdog_mailer:message]" at [site:name] ([site:url])');
    $session->fieldValueEquals('mail_body', "[site:name] ([site:url]) reported the following:\n\n[watchdog_mailer:message]\n[watchdog_mailer:php_backtrace]\n\n---\n\nProject: [site:url]\nChannel: [watchdog_mailer:channel]\nBy: [watchdog_mailer:user:display-name] (UID: [watchdog_mailer:user], IP: [watchdog_mailer:user_ip])\nAt: [watchdog_mailer:timestamp:medium]\nReferrer: [watchdog_mailer:referrer]\n\n---\nThis message was generated by Watchdog Mailer\n[watchdog_mailer:watchdog_mailer_url]");
    $session->checkboxNotChecked('php_notices');
    $session->checkboxNotChecked('channels_enable');
    $session->fieldValueEquals('channels_list', '');
    $session->checkboxNotChecked('channels_negate');
    $session->fieldValueEquals('max_emails_per_hour', 0);
  }

  /**
   * Tests the basic functionality of the settings page.
   */
  public function testBasicSettingsChange() {
    $session = $this->assertSession();
    // Go to the settings page and check if the default settings are applied.
    $this->drupalGet('/admin/config/development/logging');
    $edit = [
      'watchdog_mailer_enable' => TRUE,
      'mail_addresses' => 'foo@bar.baz',
      'mail_subject' => 'test_mail_subject',
      'mail_body' => 'test_mail_body',
      'php_notices' => TRUE,
      'max_emails_per_hour' => 100,
    ];
    $this->submitForm($edit, 'Save configuration');
    // Check saved settings.
    $session->pageTextContains('The configuration options have been saved.');
    $session->checkboxChecked('watchdog_mailer_enable');
    $session->fieldValueEquals('mail_addresses', 'foo@bar.baz');
    $session->fieldValueEquals('mail_subject', 'test_mail_subject');
    $session->fieldValueEquals('mail_body', 'test_mail_body');
    $session->checkboxChecked('php_notices');
    $session->fieldValueEquals('max_emails_per_hour', 100);
    // Check configuration.
    $this->rebuildContainer();
    $this->assertSame($edit['watchdog_mailer_enable'], $this->config('watchdog_mailer.settings')->get('watchdog_mailer_enable'));
    $this->assertSame($edit['mail_addresses'], $this->config('watchdog_mailer.settings')->get('mail_addresses'));
    $this->assertSame($edit['mail_subject'], $this->config('watchdog_mailer.settings')->get('mail_subject'));
    $this->assertSame($edit['mail_body'], $this->config('watchdog_mailer.settings')->get('mail_body'));
    $this->assertSame($edit['php_notices'], $this->config('watchdog_mailer.settings')->get('php_notices'));
    $this->assertSame($edit['max_emails_per_hour'], $this->config('watchdog_mailer.settings')->get('max_emails_per_hour'));
  }

  /**
   * Tests the basic functionality of the settings page.
   */
  public function testChannelSettings() {
    $session = $this->assertSession();
    // Go to the settings page and check if the default settings are applied.
    $this->drupalGet('/admin/config/development/logging');
    $edit = [
      'watchdog_mailer_enable' => TRUE,
      'mail_addresses' => 'another@bar.baz',
      'channels_enable' => TRUE,
      'channels_list' => 'test_channel_name',
      'channels_negate' => TRUE,
    ];
    $this->submitForm($edit, 'Save configuration');
    // Check saved settings.
    $session->pageTextContains('The configuration options have been saved.');
    $session->checkboxChecked('watchdog_mailer_enable');
    $session->fieldValueEquals('mail_addresses', 'another@bar.baz');
    $session->checkboxChecked('channels_enable');
    $session->fieldValueEquals('channels_list', 'test_channel_name');
    $session->checkboxChecked('channels_negate');
    // Check configuration.
    $this->rebuildContainer();
    $this->assertSame($edit['watchdog_mailer_enable'], $this->config('watchdog_mailer.settings')->get('watchdog_mailer_enable'));
    $this->assertSame($edit['mail_addresses'], $this->config('watchdog_mailer.settings')->get('mail_addresses'));
    $this->assertSame($edit['channels_enable'], $this->config('watchdog_mailer.settings')->get('channels_enable'));
    $this->assertSame($edit['channels_list'], $this->config('watchdog_mailer.settings')->get('channels_list'));
    $this->assertSame($edit['channels_negate'], $this->config('watchdog_mailer.settings')->get('channels_negate'));
  }

  /**
   * Test settings form error functions.
   */
  public function testSettingsFormErrors() {
    $this->drupalGet('/admin/config/development/logging');
    // Test single invalid email-address.
    $edit = [
      'watchdog_mailer_enable' => TRUE,
      'mail_addresses' => 'invalid_email_address',
    ];
    $this->submitForm($edit, 'Save configuration');
    $this->assertSession()->statusMessageContains('invalid_email_address is not a valid email address.', 'error');
    // Test multiple invalid email-address.
    $edit = [
      'watchdog_mailer_enable' => TRUE,
      'mail_addresses' => 'example@example.com, foobar.baz_invalid',
    ];
    $this->submitForm($edit, 'Save configuration');
    $this->assertSession()->statusMessageContains('foobar.baz_invalid is not a valid email address.', 'error');
  }

}
