<?php

declare(strict_types = 1);

namespace Drupal\Tests\watchdog_mailer\FunctionalJavascript;

use Drupal\FunctionalJavascriptTests\WebDriverTestBase;

/**
 * Tests the #states functionality of the Watchdog Mailer module settings form.
 *
 * @group watchdog_mailer
 */
final class SettingsFormStatesTest extends WebDriverTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = ['watchdog_mailer'];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'claro';

  /**
   * A user with admin permissions.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $adminUser;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->adminUser = $this->drupalCreateUser(['administer site configuration']);
    $this->drupalLogin($this->adminUser);
  }

  /**
   * Test callback.
   */
  public function testSettingsFormStates(): void {
    $this->drupalGet('/admin/config/development/logging');
    $page = $this->getSession()->getPage();
    $watchdog_mailer_enable = $page->findField('watchdog_mailer_enable');
    $watchdog_mailer = $page->find('css', '#edit-watchdog-mailer');
    $mail_addresses = $page->findField('mail_addresses');
    $mail_subject = $page->findField('mail_subject');
    $mail_body = $page->findField('mail_body');
    $php_notices = $page->findField('php_notices');
    $channels_enable = $page->findField('channels_enable');
    $channels_list = $page->findField('channels_list');
    $channels_negate = $page->findField('channels_negate');
    $max_emails_per_hour = $page->findField('max_emails_per_hour');
    // Test initial condition.
    $this->assertTrue($watchdog_mailer_enable->isVisible());
    $this->assertFalse($watchdog_mailer->isVisible());
    // Enable basic watchdog mailer functionality.
    $watchdog_mailer_enable->check();
    $this->assertTrue($watchdog_mailer_enable->isVisible());
    $this->assertTrue($watchdog_mailer->isVisible());
    $this->assertTrue($mail_addresses->isVisible());
    $this->assertTrue($mail_subject->isVisible());
    $this->assertTrue($mail_body->isVisible());
    $this->assertTrue($php_notices->isVisible());
    $this->assertTrue($channels_enable->isVisible());
    $this->assertFalse($channels_list->isVisible());
    $this->assertFalse($channels_negate->isVisible());
    $this->assertTrue($max_emails_per_hour->isVisible());
    // Enable PHP notices.
    $php_notices->check();
    $this->assertFalse($channels_enable->isVisible());
    $this->assertFalse($channels_list->isVisible());
    $this->assertFalse($channels_negate->isVisible());
    $php_notices->uncheck();
    // Enable channel lists.
    $channels_enable->check();
    $this->assertTrue($channels_list->isVisible());
    $this->assertTrue($channels_negate->isVisible());
  }

}
